--- old/usr/src/uts/common/inet/ip/inet_ntop.c	2012-09-26 12:51:45.924618132 +0000
+++ new/usr/src/uts/common/inet/ip/inet_ntop.c	2012-09-26 12:51:44.759663909 +0000
@@ -23,8 +23,9 @@
  * Copyright 2004 Sun Microsystems, Inc.  All rights reserved.
  * Use is subject to license terms.
  */
-
-#pragma ident	"%Z%%M%	%I%	%E% SMI"
+/*
+ * Copyright 2012 Nexenta Systems, Inc. All rights reserved.
+ */
 
 #include <sys/types.h>
 #include <sys/cmn_err.h>
@@ -233,14 +234,16 @@
 	char *end;
 	long byte;
 	int i;
-	ipaddr_t addr = 0;
+	uint8_t *addr = (uint8_t *)addrp;
+
+	*addrp = 0;
 
 	for (i = 0; i < 4; i++) {
 		if (ddi_strtol(cp, &end, 10, &byte) != 0 || byte < 0 ||
 		    byte > 255) {
 			return (0);
 		}
-		addr = (addr << 8) | (uint8_t)byte;
+		addr[i] = (uint8_t)byte;
 		if (i < 3) {
 			if (*end != '.') {
 				return (0);
@@ -251,7 +254,7 @@
 			cp = end;
 		}
 	}
-	*addrp = addr;
+
 	return (1);
 }
 
@@ -264,7 +267,7 @@
  *            -1 when af is not AF_INET or AF_INET6
  */
 int
-inet_pton(int af, char *inp, void *outp)
+m_inet_pton(int af, char *inp, void *outp, int revert)
 {
 	int i;
 	long byte;
@@ -272,7 +275,13 @@
 
 	switch (af) {
 	case AF_INET:
-		return (str2inet_addr(inp, (ipaddr_t *)outp));
+		if (str2inet_addr(inp, (ipaddr_t *)outp)) {
+			if (! revert)
+				*(uint32_t *)outp = ntohl(*(uint32_t *)outp);
+			return (1);
+		} else {
+			return (0);
+		}
 	case AF_INET6: {
 		union v6buf_u {
 			uint16_t v6words_u[8];
@@ -316,7 +325,11 @@
 			if (byte < 0 || byte > 0x0ffff) {
 				return (0);
 			}
-			v6buf.v6words_u[i] = (uint16_t)byte;
+			if (revert) {
+				v6buf.v6words_u[i] = htons((uint16_t)byte);
+			} else {
+				v6buf.v6words_u[i] = (uint16_t)byte;
+			}
 			if (*end == NULL || i == 7) {
 				inp = end;
 				break;
@@ -388,3 +401,24 @@
 	}	/* switch */
 	return (-1);	/* return -1 for default case */
 }
+
+
+int
+_inet_pton(int af, char *inp, void *outp)
+{
+	return (m_inet_pton(af, inp, outp, 1));
+}
+
+/*
+ * We need this inet_pton to preserve compatibility with old closed binaries.
+ * Earlier, inet_pton returned address in hardware native order,
+ * not in network one. (See http://www.illumos.org/issue/3105).
+ * Having fixed that, we still need to support binaries, that use bad inet_pton
+ * and reverse returned address manually. All new inet_pton calls will be
+ * redirected to _inet_pton with #define in the header file.
+ */
+int
+inet_pton(int af, char *inp, void *outp)
+{
+	return (m_inet_pton(af, inp, outp, 0));
+}
